package be.cylab.java.wowa.training;

import java.util.logging.Logger;

/**
 * Parameters for the Trainer object.
 */
public class TrainerParameters {
    /**
     * Constant for choosing the selection method.
     * SELECTION_METHODRWS for the roulette wheel selection method
     * SELECTION_METHOD_TOS for the tournament selection
     */
    public static final int SELECTION_METHOD_RWS = 374;
    public static final int SELECTION_METHOD_TOS = 942;


    private Logger logger;
    private int population_size;
    private int crossover_rate;
    private int mutation_rate;
    private int selection_method;

    private int max_generation_number;
    private float trigger_distance;


    /**
     * Contains parameters for the WOWA trainer.
     *
     * @param population_size       Size of the population for the
     *                              Genetic Algorithm (greater than 2)
     * @param crossover_rate        Crossover rate for the genetic algorithm.
     *                              Percentage of the population kept
     *                              to generate the next generation (1-99)
     * @param mutation_rate         Mutation rate for the genetic algorithm.
     *                              percentage of genes mutated in the
     *                              next generation (1-99)
     * @param selection_method      Method to select parents for the next
     *                              generation. RWS or TOS
     * @param max_generation_number Max generation number before
     *                              the algorithm stop
     */
    public TrainerParameters(
            //Logger logger,
            int population_size,
            int crossover_rate,
            int mutation_rate,
            int selection_method,
            int max_generation_number

    ) {
        //this.setLogger(logger);
        this.setPopulationSize(population_size);
        this.setCrossoverRate(crossover_rate);
        this.setMutationRate(mutation_rate);
        this.setSelectionMethod(selection_method);
        this.setMaxGenerationNumber(max_generation_number);

    }

    /**
     * Getter parent_number
     * @return int, number of parents. Depends on the crossover rate
     */
    public int getNumberParents() {
        int nbreParents = Math.round(this.getPopulationSize() * (1 - this.getCrossoverRate() / 100));
        if (nbreParents % 2 == 1) {
            nbreParents++;
        }
        return nbreParents;
    }

    /**
     * @return Logger
     */
    public Logger getLogger() {
        return logger;
    }


    /**
     * Getter for population_size
     * @return int
     */
    public int getPopulationSize() {
        return population_size;
    }

    /**
     * Setter population_size
     * @param population_size, must be greater than 2
     * @throws ArithmeticException
     */
    private void setPopulationSize(int population_size) throws ArithmeticException {
        if (population_size < 2) {
            throw new ArithmeticException("Population must be higher than 2");
        }
        this.population_size = population_size;
    }

    /**
     * Getter for corssover rate
     * @return int
     */
    private int getCrossoverRate() {
        return crossover_rate;
    }

    /**
     * Setter for corssover rate
     * @param crossover_rate
     */
    private void setCrossoverRate(int crossover_rate) {
        this.crossover_rate = crossover_rate;
    }

    /**
     * Getter for mutation rate
     * @return int
     */
    public int getMutationRate() {
        return mutation_rate;
    }

    /**
     * Setter for mutation rate
     * @param mutation_rate
     */
    private void setMutationRate(int mutation_rate) {
        this.mutation_rate = mutation_rate;
    }

    /**
     * Getter for selection method
     * @return int
     */
    public int getSelectionMethod() {
        return selection_method;
    }

    /**
     * Setter for selection method
     * @param selection_method int, must be RWS or TOS method
     * @throws IllegalArgumentException
     */
    private void setSelectionMethod(int selection_method) {
        if (selection_method != this.SELECTION_METHOD_RWS &&
                selection_method != this.SELECTION_METHOD_TOS) {
            throw new IllegalArgumentException("Not a valid selection method");
        }
        this.selection_method = selection_method;
    }

    /**
     * Getter for max generation number
     * @return int
     */
    public int getMaxGenerationNumber() {
        return max_generation_number;
    }

    /**
     * Setter for max generation number
     * @param max_generation_number
     */
    private void setMaxGenerationNumber(int max_generation_number) {
        this.max_generation_number = max_generation_number;
    }

    /**
     * Getter for trigger distance
     * @return double
     */
    public double getTriggerDistance() {
        return trigger_distance;
    }

}
